unit uMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Generics.Collections;

type
  TForm1 = class(TForm)
    btnRefresh: TButton;
    cbPalWindows: TComboBox;
    lblTitle: TLabel;
    lblMainHwnd: TLabel;
    lblIncomming: TLabel;
    lblOutgoing: TLabel;
    lblNicklist: TLabel;
    procedure btnRefreshClick(Sender: TObject);
    procedure cbPalWindowsSelect(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


  { Record for holding all our paltalk handles }
type
  TWindowInfo = record
    sMain: array [0 .. 255] Of Char;
    hMain: HWND;
    hIncomming: HWND;
    hOutgoing: HWND;
    hNicklist: HWND;
  end;

var
  Form1: TForm1;
  { List for holding all our records }
  WindowList: TList<TWindowInfo>;

implementation

{$R *.dfm}

{ Set all record values to nil }
function ZeroOutWinInfo(rWinInfo: TWindowInfo): TWindowInfo;
begin
  FillChar(rWinInfo.sMain, SizeOf(rWinInfo.sMain), #0);
  rWinInfo.hMain := 0;
  rWinInfo.hIncomming := 0;
  rWinInfo.hOutgoing := 0;
  rWinInfo.hNicklist := 0;
  Result := rWinInfo;
end;

{ Get Window title based on Hwnd }
function GetWindowTitle(HWND: HWND): string;
begin
  SetLength(Result, 255);
  SetLength(Result, GetWindowText(HWND, PChar(Result), 255));
end;

{ Get the Classname based on Hwnd }
function GetWindowClass(HWND: HWND): string;
begin
  SetLength(Result, 255);
  SetLength(Result, GetClassName(HWND, PChar(Result), 255));
end;

{ EnumChildWindows Callback }
function EnumChildProc(aHwnd: HWND; Param: lParam): Boolean; stdcall;
var
  WindowList: TList<TWindowInfo>;
  WindowInfo: TWindowInfo;
begin

  WindowList := TList<TWindowInfo>(Param);

  if IsWindowVisible(aHwnd) = True then
  begin
    WindowInfo := WindowList.Items[Pred(WindowList.Count)];

    case GetDlgCtrlID(aHwnd) of
      202:
        begin
          WindowInfo.hIncomming := aHwnd;
          WindowList.Items[Pred(WindowList.Count)] := WindowInfo;
        end;
      203:
        begin
          WindowInfo.hOutgoing := aHwnd;
          WindowList.Items[Pred(WindowList.Count)] := WindowInfo;
        end;
      1789:
        begin
          WindowInfo.hNicklist := aHwnd;
          WindowList.Items[Pred(WindowList.Count)] := WindowInfo;
        end;
    end;
  end;
  Result := True;
end;

{ EnumWindows Callback }
function EnumWinProc(aHwnd: HWND; Param: lParam): Boolean; stdcall;
var
  WindowList: TList<TWindowInfo>;
  WindowInfo: TWindowInfo;
begin
  WindowList := TList<TWindowInfo>(Param);

  if GetWindowClass(aHwnd) = 'DlgGroupChat Window Class' then
  begin
    WindowInfo := ZeroOutWinInfo(WindowInfo);
    WindowInfo.hMain := aHwnd;
    GetWindowText(aHwnd, WindowInfo.sMain, 255);
    WindowList.Add(WindowInfo);
    EnumChildWindows(aHwnd, @EnumChildProc, Param);
  end;
  Result := True;

end;

{ Update Combobox With Rooms }
procedure TForm1.btnRefreshClick(Sender: TObject);
var
  WindowInfo: TWindowInfo;
begin
  WindowList.Clear;
  cbPalWindows.Clear;
  cbPalWindows.Text := 'Room Selector';
  EnumWindows(@EnumWinProc, lParam(WindowList));
  for WindowInfo in WindowList do
    cbPalWindows.Items.Add(WindowInfo.sMain);
end;

{ On select display our handles for current window }
procedure TForm1.cbPalWindowsSelect(Sender: TObject);
begin
  lblTitle.Caption := 'Title: ' + WindowList.Items[cbPalWindows.ItemIndex].sMain;
  lblMainHwnd.Caption := 'MainHwnd: ' +
    IntToStr(WindowList.Items[cbPalWindows.ItemIndex].hMain);
  lblIncomming.Caption := 'IncommingHwnd: ' +
    IntToStr(WindowList.Items[cbPalWindows.ItemIndex].hIncomming);
  lblOutgoing.Caption := 'OutgoingHwnd: ' +
    IntToStr(WindowList.Items[cbPalWindows.ItemIndex].hOutgoing);
  lblNicklist.Caption := 'NicklistHwnd: ' +
    IntToStr(WindowList.Items[cbPalWindows.ItemIndex].hNicklist);
end;

{ Create WindowList note: add "ReportMemoryLeaksOnShutdown := True;" to projects source }
procedure TForm1.FormCreate(Sender: TObject);
begin
  WindowList := TList<TWindowInfo>.Create;
end;

{ Make sure our data has been disposed of }
procedure TForm1.FormDestroy(Sender: TObject);
begin
  WindowList.Free;
end;


end.
